/// SPDX-License-Identifier: GPL-3.0-or-later
/// SPDX-FileCopyrightText: Copyright © 2016-2024 Tony Garnock-Jones <tonyg@leastfixedpoint.com>

import { Bytes } from '@preserves/core';
import * as node_crypto from 'crypto';

export function _btoa(s: string): string {
    try {
        return btoa(s);
    } catch (e) {
        return Buffer.from(s).toString('base64');
    }
}

function _finish(hexOutput: boolean, buf: Uint8Array): string {
    if (hexOutput) {
        return Bytes.from(buf).toHex();
    } else {
        return _btoa(String.fromCharCode.apply(null, buf as unknown as number[])).replace(/=/g,'');
    }
}

export const randomId =
    (typeof crypto !== 'undefined' && 'getRandomValues' in crypto)
    ? ((byteCount: number, hexOutput: boolean = false): string => {
        const buf = new Uint8Array(byteCount);
        crypto.getRandomValues(buf);
        return _finish(hexOutput, buf);
    })
    : ((byteCount: number, hexOutput: boolean = false): string => {
        let buf = node_crypto.randomBytes(byteCount);
        return _finish(hexOutput, buf);
    });
